﻿using System;
using System.Configuration;
using System.Data;
using System.Data.Common;
using System.Data.SqlClient;
using DayPilot.Web.Ui;
using DayPilot.Web.Ui.Recurrence;

namespace Data
{
    public class DataManager
    {

        private DbDataAdapter CreateDataAdapter(string select)
        {
            DbDataAdapter da = Factory.CreateDataAdapter();
            da.SelectCommand = CreateCommand(select);
            return da;
        }

        public DataTable GetTasks()
        {
            var da = CreateDataAdapter("select * from [Task] order by [TaskStart]");
            DataTable dt = new DataTable();
            da.Fill(dt);
            return dt;
        }


        #region Helper methods
        private string ConnectionString
        {
            get { return Db.ConnectionString(); }
        }

        private DbProviderFactory Factory
        {
            get { return Db.Factory(); }
        }

        private DbConnection CreateConnection()
        {
            DbConnection connection = Factory.CreateConnection();
            connection.ConnectionString = ConnectionString;
            return connection;
        }

        private DbCommand CreateCommand(string text)
        {
            DbCommand command = Factory.CreateCommand();
            command.CommandText = text;
            command.Connection = CreateConnection();

            return command;
        }

        private DbCommand CreateCommand(string text, DbConnection connection)
        {
            DbCommand command = Factory.CreateCommand();
            command.CommandText = text;
            command.Connection = connection;

            return command;
        }

        private void AddParameterWithValue(DbCommand cmd, string name, object value)
        {
            var parameter = Factory.CreateParameter();
            parameter.Direction = ParameterDirection.Input;
            parameter.ParameterName = name;
            parameter.Value = value;
            cmd.Parameters.Add(parameter);
        }

        private int GetIdentity(DbConnection c)
        {
            var cmd = CreateCommand(Db.IdentityCommand(), c);
            return Convert.ToInt32(cmd.ExecuteScalar());
        }

        #endregion

        public DateTime? GetStart()
        {
            // dumb and inefficient implementation, replace with TOP 1
            var da = CreateDataAdapter("select * from [Task] order by [TaskStart]");
            DataTable dt = new DataTable();
            da.Fill(dt);
            if (dt.Rows.Count > 0)
            {
                return Convert.ToDateTime(dt.Rows[0]["TaskStart"]);
            }
            return null;
        }

        public DateTime? GetEnd()
        {
            // dumb and inefficient implementation, replace with TOP 1
            var da = CreateDataAdapter("select * from [Task] order by [TaskStart] desc");
            DataTable dt = new DataTable();
            da.Fill(dt);
            if (dt.Rows.Count > 0)
            {
                return Convert.ToDateTime(dt.Rows[0]["TaskEnd"]);
            }
            return null;
        }

        public void DeleteTask(int id)
        {
            using (var con = CreateConnection())
            {
                con.Open();

                var cmd = CreateCommand("delete from [Task] where [TaskId] = @id", con);
                AddParameterWithValue(cmd, "id", id);
                cmd.ExecuteNonQuery();

            }
        }

        public void MoveTask(int id, DateTime start, DateTime end)
        {
            using (var con = CreateConnection())
            {
                con.Open();

                var cmd = CreateCommand("update [Task] set [TaskStart] = @start, [TaskEnd] = @end where [TaskId] = @id", con);
                AddParameterWithValue(cmd, "id", id);
                AddParameterWithValue(cmd, "start", start);
                AddParameterWithValue(cmd, "end", end);
                cmd.ExecuteNonQuery();
            }
        }

        public void CreateTask(DateTime start, DateTime end, string name)
        {
            using (DbConnection con = CreateConnection())
            {
                con.Open();

                var cmd = CreateCommand("insert into [Task] ([TaskStart], [TaskEnd], [TaskName]) values (@start, @end, @name)", con);
                AddParameterWithValue(cmd, "start", start);
                AddParameterWithValue(cmd, "end", end);
                AddParameterWithValue(cmd, "name", name);
                cmd.ExecuteNonQuery();

            }
        }

        public DataRow GetTask(int id)
        {
            var da = CreateDataAdapter("select * from [Task] where [TaskId] = @id");
            AddParameterWithValue(da.SelectCommand, "id", id);
            DataTable dt = new DataTable();
            da.Fill(dt);
            if (dt.Rows.Count > 0)
            {
                return dt.Rows[0];
            }
            return null;
        }

        public void UpdateTask(int id, DateTime start, DateTime end, string name)
        {
            using (var con = CreateConnection())
            {
                con.Open();

                var cmd = CreateCommand("update [Task] set [TaskStart] = @start, [TaskEnd] = @end, [TaskName] = @name where [TaskId] = @id", con);
                AddParameterWithValue(cmd, "id", id);
                AddParameterWithValue(cmd, "start", start);
                AddParameterWithValue(cmd, "end", end);
                AddParameterWithValue(cmd, "name", name);
                cmd.ExecuteNonQuery();
            }
        }
    }
}