﻿using System;
using System.Collections;
using System.Data;
using System.Web;
using System.Web.Script.Services;
using System.Web.Services;
using Data;
using DayPilot.Utils;
using DayPilot.Web.Ui;
using DayPilot.Web.Ui.Enums;
using DayPilot.Web.Ui.Events;
using DayPilot.Web.Ui.Events.Scheduler;
using Newtonsoft.Json;

public partial class _Default : System.Web.UI.Page
{
    private DataTable tasks;

    protected void Page_Load(object sender, EventArgs e)
    {
        if (!IsPostBack)
        {
            DayPilotScheduler1.TimeHeaders.Clear();
            DayPilotScheduler1.TimeHeaders.Add(new TimeHeader(GroupByEnum.Month));
            DayPilotScheduler1.TimeHeaders.Add(new TimeHeader(GroupByEnum.Week));
            DayPilotScheduler1.TimeHeaders.Add(new TimeHeader(GroupByEnum.Day));

            LoadResources();
            UpdateScheduler();

            DayPilotScheduler1.SetScrollX(new DataManager().GetStart() ?? DateTime.Today);
        }
    }

    private void UpdateScheduler()
    {
        tasks = new DataManager().GetTasks();
        var start = new DateTime(2021, 1, 1);
        var end = new DateTime(2022, 1, 1);

        DayPilotScheduler1.StartDate = start;
        DayPilotScheduler1.Days = (int)Math.Ceiling((end - start).TotalDays);
        DayPilotScheduler1.DataSource = new DataManager().GetTasks();
        DayPilotScheduler1.DataBind();
        DayPilotScheduler1.Update();
    }

    private void LoadResources()
    {
        DataTable locations = new DataManager().GetTasks();
        DayPilotScheduler1.Resources.Clear();
        DayPilotScheduler1.Resources.Add("New Task", "NEW");
        foreach (DataRow dr in locations.Rows)
        {
            DayPilotScheduler1.Resources.Add((string)dr["TaskName"], Convert.ToString(dr["TaskId"]));
        }
    }

    protected void DayPilotCalendar1_EventMenuClick(object sender, EventMenuClickEventArgs e)
    {
        switch (e.Command)
        {
            case "Delete":
                int id = e.Recurrent ? Convert.ToInt32(e.RecurrentMasterId) : Convert.ToInt32(e.Value);
                new DataManager().DeleteTask(id);
                UpdateScheduler();
                LoadResources();
                DayPilotScheduler1.Update(CallBackUpdateType.Full);
                break;


        }
    }

    protected void DayPilotCalendar1_EventResize(object sender, EventResizeEventArgs e)
    {
        int id = e.Recurrent ? Convert.ToInt32(e.RecurrentMasterId) : Convert.ToInt32(e.Value);
        new DataManager().MoveTask(id, e.NewStart, e.NewEnd);
        LoadResources(); // update order
        UpdateScheduler();
    }

    protected void DayPilotCalendar1_Command(object sender, CommandEventArgs e)
    {
        switch (e.Command)
        {
            case "refresh":
                UpdateScheduler();
                LoadResources();
                DayPilotScheduler1.Update(CallBackUpdateType.Full);
                break;

        }
    }

    protected void DayPilotCalendar1_BeforeEventRender(object sender, BeforeEventRenderEventArgs e)
    {
        e.InnerHTML = Server.HtmlEncode(e.Text);
        e.EventMoveVerticalEnabled = false;
        e.StaticBubbleHTML = String.Format("<b>{0}</b><br/>Start: {1}<br/>End: {2}", e.Text, e.Start, e.End);

    }

    protected void DayPilotCalendar1_EventMove(object sender, EventMoveEventArgs e)
    {
        int id = e.Recurrent ? Convert.ToInt32(e.RecurrentMasterId) : Convert.ToInt32(e.Value);
        new DataManager().MoveTask(id, e.NewStart, e.NewEnd);
        LoadResources(); // update order
        UpdateScheduler();
    }
    protected void DayPilotScheduler1_BeforeCellRender(object sender, BeforeCellRenderEventArgs e)
    {
    }

    protected void DayPilotScheduler1_BeforeTimeHeaderRender(object sender, BeforeTimeHeaderRenderEventArgs e)
    {
        if (e.Level == 1)
        {
            e.InnerHTML = String.Format("Week {0}", Week.WeekNrISO8601(e.Start));
        }
    }

    [WebMethod(EnableSession = true), ScriptMethod(ResponseFormat = ResponseFormat.Json)]
    public static string CreateTask(DateTime start, DateTime end, string text)
    {
        new DataManager().CreateTask(start, end, text);
        Hashtable result = new Hashtable();
        result["status"] = "OK";
        return JsonConvert.SerializeObject(result);
    }


    [WebMethod(EnableSession = true), ScriptMethod(ResponseFormat = ResponseFormat.Json)]
    public static string UpdateTask(int id, DateTime start, DateTime end, string text)
    {
        new DataManager().UpdateTask(id, start, end, text);
        Hashtable result = new Hashtable();
        result["status"] = "OK";
        return JsonConvert.SerializeObject(result);
    }

}
