﻿Imports System
Imports System.Configuration
Imports System.Data
Imports System.Data.Common
Imports System.Data.SqlClient
Imports DayPilot.Web.Ui
Imports DayPilot.Web.Ui.Recurrence

Namespace Data
	Public Class DataManager

		Private Function CreateDataAdapter(ByVal [select] As String) As DbDataAdapter
			Dim da As DbDataAdapter = Factory.CreateDataAdapter()
			da.SelectCommand = CreateCommand([select])
			Return da
		End Function

		Public Function GetTasks() As DataTable
			Dim da = CreateDataAdapter("select * from [Task] order by [TaskStart]")
			Dim dt As New DataTable()
			da.Fill(dt)
			Return dt
		End Function


		#Region "Helper methods"
		Private ReadOnly Property ConnectionString() As String
			Get
				Return Db.ConnectionString()
			End Get
		End Property

		Private ReadOnly Property Factory() As DbProviderFactory
			Get
				Return Db.Factory()
			End Get
		End Property

		Private Function CreateConnection() As DbConnection
			Dim connection As DbConnection = Factory.CreateConnection()
			connection.ConnectionString = ConnectionString
			Return connection
		End Function

		Private Function CreateCommand(ByVal text As String) As DbCommand
			Dim command As DbCommand = Factory.CreateCommand()
			command.CommandText = text
			command.Connection = CreateConnection()

			Return command
		End Function

		Private Function CreateCommand(ByVal text As String, ByVal connection As DbConnection) As DbCommand
			Dim command As DbCommand = Factory.CreateCommand()
			command.CommandText = text
			command.Connection = connection

			Return command
		End Function

		Private Sub AddParameterWithValue(ByVal cmd As DbCommand, ByVal name As String, ByVal value As Object)
			Dim parameter = Factory.CreateParameter()
			parameter.Direction = ParameterDirection.Input
			parameter.ParameterName = name
			parameter.Value = value
			cmd.Parameters.Add(parameter)
		End Sub

		Private Function GetIdentity(ByVal c As DbConnection) As Integer
			Dim cmd = CreateCommand(Db.IdentityCommand(), c)
			Return Convert.ToInt32(cmd.ExecuteScalar())
		End Function

		#End Region

		Public Function GetStart() As Date?
			' dumb and inefficient implementation, replace with TOP 1
			Dim da = CreateDataAdapter("select * from [Task] order by [TaskStart]")
			Dim dt As New DataTable()
			da.Fill(dt)
			If dt.Rows.Count > 0 Then
				Return Convert.ToDateTime(dt.Rows(0)("TaskStart"))
			End If
			Return Nothing
		End Function

		Public Function GetEnd() As Date?
			' dumb and inefficient implementation, replace with TOP 1
			Dim da = CreateDataAdapter("select * from [Task] order by [TaskStart] desc")
			Dim dt As New DataTable()
			da.Fill(dt)
			If dt.Rows.Count > 0 Then
				Return Convert.ToDateTime(dt.Rows(0)("TaskEnd"))
			End If
			Return Nothing
		End Function

		Public Sub DeleteTask(ByVal id As Integer)
			Using con = CreateConnection()
				con.Open()

				Dim cmd = CreateCommand("delete from [Task] where [TaskId] = @id", con)
				AddParameterWithValue(cmd, "id", id)
				cmd.ExecuteNonQuery()

			End Using
		End Sub

		Public Sub MoveTask(ByVal id As Integer, ByVal start As Date, ByVal [end] As Date)
			Using con = CreateConnection()
				con.Open()

				Dim cmd = CreateCommand("update [Task] set [TaskStart] = @start, [TaskEnd] = @end where [TaskId] = @id", con)
				AddParameterWithValue(cmd, "id", id)
				AddParameterWithValue(cmd, "start", start)
				AddParameterWithValue(cmd, "end", [end])
				cmd.ExecuteNonQuery()
			End Using
		End Sub

		Public Sub CreateTask(ByVal start As Date, ByVal [end] As Date, ByVal name As String)
			Using con As DbConnection = CreateConnection()
				con.Open()

				Dim cmd = CreateCommand("insert into [Task] ([TaskStart], [TaskEnd], [TaskName]) values (@start, @end, @name)", con)
				AddParameterWithValue(cmd, "start", start)
				AddParameterWithValue(cmd, "end", [end])
				AddParameterWithValue(cmd, "name", name)
				cmd.ExecuteNonQuery()

			End Using
		End Sub

		Public Function GetTask(ByVal id As Integer) As DataRow
			Dim da = CreateDataAdapter("select * from [Task] where [TaskId] = @id")
			AddParameterWithValue(da.SelectCommand, "id", id)
			Dim dt As New DataTable()
			da.Fill(dt)
			If dt.Rows.Count > 0 Then
				Return dt.Rows(0)
			End If
			Return Nothing
		End Function

		Public Sub UpdateTask(ByVal id As Integer, ByVal start As Date, ByVal [end] As Date, ByVal name As String)
			Using con = CreateConnection()
				con.Open()

				Dim cmd = CreateCommand("update [Task] set [TaskStart] = @start, [TaskEnd] = @end, [TaskName] = @name where [TaskId] = @id", con)
				AddParameterWithValue(cmd, "id", id)
				AddParameterWithValue(cmd, "start", start)
				AddParameterWithValue(cmd, "end", [end])
				AddParameterWithValue(cmd, "name", name)
				cmd.ExecuteNonQuery()
			End Using
		End Sub
	End Class
End Namespace